@@ -22,7 +22,8 @@ from .consts import LookupConventions as const
 from .exceptions import APIKeyMissingError
 
 UTC = pytz.UTC
-timestamp_now = datetime.utcnow().replace(tzinfo=UTC)
+
+REDIS_LUA_DEL_SCRIPT = "local keys = redis.call('keys', ARGV[1]) \n for i=1,#keys,20000 do \n redis.call('del', unpack(keys, i, math.min(i+19999, #keys))) \n end \n return keys"
 
 if sys.version_info < (2, 7,):
     class NullHandler(logging.Handler):
@@ -224,18 +225,26 @@ class LookupLib(object):
 
     def _push_dict_to_redis(self, push_dict, redis_prefix, name):
         r = self._redis
+        pipe = r.pipeline()
+        pipe.eval(REDIS_LUA_DEL_SCRIPT, 0, redis_prefix + name)
+
         for i in push_dict:
             json_data = self._serialize_data(push_dict[i])
-            r.delete(redis_prefix + name + str(i))
-            r.set(redis_prefix + name + str(i), json_data)
+            pipe.set(redis_prefix + name + str(i), json_data)
+
+        pipe.execute()
         return True
 
     def _push_dict_index_to_redis(self, index_dict, redis_prefix, name):
         r = self._redis
+        pipe = r.pipeline()
+        pipe.eval(REDIS_LUA_DEL_SCRIPT, 0, redis_prefix + name)
+
         for i in index_dict:
-            r.delete(redis_prefix + name + str(i))
             for el in index_dict[i]:
-                r.sadd(redis_prefix + name + str(i), el)
+                pipe.sadd(redis_prefix + name + str(i), el)
+
+        pipe.execute()
         return True
 
 
@@ -319,7 +328,7 @@ class LookupLib(object):
         return new_dict
 
 
-    def lookup_callsign(self, callsign=None, timestamp=timestamp_now):
+    def lookup_callsign(self, callsign=None, timestamp=None):
         """
         Returns lookup data if an exception exists for a callsign
 
@@ -364,6 +373,8 @@ class LookupLib(object):
 
         """
         callsign = callsign.strip().upper()
+        if timestamp is None:
+            timestamp = datetime.utcnow().replace(tzinfo=UTC)
 
         if self._lookuptype == "clublogapi":
             callsign_data =  self._lookup_clublogAPI(callsign=callsign, timestamp=timestamp, apikey=self._apikey)
@@ -482,7 +493,7 @@ class LookupLib(object):
         raise KeyError
 
 
-    def lookup_prefix(self, prefix, timestamp=timestamp_now):
+    def lookup_prefix(self, prefix, timestamp=None):
         """
         Returns lookup data of a Prefix
 
@@ -524,6 +535,8 @@ class LookupLib(object):
         """
 
         prefix = prefix.strip().upper()
+        if timestamp is None:
+            timestamp = datetime.utcnow().replace(tzinfo=UTC)
 
         if self._lookuptype == "clublogxml" or self._lookuptype == "countryfile":
 
@@ -537,7 +550,7 @@ class LookupLib(object):
         # no matching case
         raise KeyError
 
-    def is_invalid_operation(self, callsign, timestamp=datetime.utcnow().replace(tzinfo=UTC)):
+    def is_invalid_operation(self, callsign, timestamp=None):
         """
         Returns True if an operations is known as invalid
 
@@ -577,6 +590,8 @@ class LookupLib(object):
         """
 
         callsign = callsign.strip().upper()
+        if timestamp is None:
+            timestamp = datetime.utcnow().replace(tzinfo=UTC)
 
         if self._lookuptype == "clublogxml":
 
@@ -624,7 +639,7 @@ class LookupLib(object):
         raise KeyError
 
 
-    def lookup_zone_exception(self, callsign, timestamp=datetime.utcnow().replace(tzinfo=UTC)):
+    def lookup_zone_exception(self, callsign, timestamp=None):
         """
         Returns a CQ Zone if an exception exists for the given callsign
 
@@ -659,6 +674,8 @@ class LookupLib(object):
         """
 
         callsign = callsign.strip().upper()
+        if timestamp is None:
+            timestamp = datetime.utcnow().replace(tzinfo=UTC)
 
         if self._lookuptype == "clublogxml":
 
@@ -672,7 +689,7 @@ class LookupLib(object):
         #no matching case
         raise KeyError
 
-    def _lookup_clublogAPI(self, callsign=None, timestamp=timestamp_now, url="https://secure.clublog.org/dxcc", apikey=None):
+    def _lookup_clublogAPI(self, callsign=None, timestamp=None, url="https://secure.clublog.org/dxcc", apikey=None):
         """ Set up the Lookup object for Clublog Online API
         """
 
@@ -686,6 +703,9 @@ class LookupLib(object):
             "call" : callsign
         }
 
+        if timestamp is None:
+            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+
         if sys.version_info.major == 3:
             encodeurl = url + "?" + urllib.parse.urlencode(params)
         else:
@@ -986,6 +1006,9 @@ class LookupLib(object):
         self._invalid_operations_index = cty_dict["invalid_operations_index"]
         self._zone_exceptions_index = cty_dict["zone_exceptions_index"]
 
+        if self._download:
+            self._cleanup_download_artifact(cty_file)
+
         return True
 
     def _load_countryfile(self,
@@ -1020,6 +1043,9 @@ class LookupLib(object):
         self._callsign_exceptions_index = cty_dict["exceptions_index"]
         self._prefixes_index = cty_dict["prefixes_index"]
 
+        if self._download:
+            self._cleanup_download_artifact(cty_file)
+
         return True
 
     def _download_file(self, url, apikey=None):
@@ -1081,6 +1107,19 @@ class LookupLib(object):
 
         return cty_file_path
 
+    def _cleanup_download_artifact(self, filename):
+        """
+        Delete the downloaded files which are not necessary anymore
+
+        Args:
+            filename (string): absolute path to the download artifact
+        """
+
+        try:
+            os.remove(filename)
+        except:
+            self._logger.warning("unable delete the download artifact: %s", _download_file)
+
     def _extract_clublog_header(self, cty_xml_filename):
         """
         Extract the header of the Clublog XML File
@@ -1475,8 +1514,7 @@ class LookupLib(object):
         Deserialize a JSON into a dictionary
         """
 
-        my_dict = json.loads(json_data.decode('utf8').replace("'", '"'),
-            encoding='UTF-8')
+        my_dict = json.loads(json_data.decode('utf8'), encoding='UTF-8')
 
         for item in my_dict:
             if item == const.ADIF:
